﻿using System;

namespace KinectAPI
{
    /// <summary>
    /// Represents a point's coordinates.
    /// </summary>
    public class PositionEventArgs : EventArgs
    {
        #region Properties

        /// <summary>
        /// Horizontal coordinate.
        /// </summary>
        public float X { get; set; }

        /// <summary>
        /// Vertical coordinate.
        /// </summary>
        public float Y { get; set; }

        /// <summary>
        /// Depth coordinate.
        /// </summary>
        public float Z { get; set; }

        #endregion

        #region Constructors

        /// <summary>
        /// Creates a new instance of PositionEventArgs specifying X and Y coordinates.
        /// </summary>
        /// <param name="x">X-axis value.</param>
        /// <param name="y">Y-axis value.</param>
        public PositionEventArgs(float x, float y)
        {
            X = x;
            Y = y;
        }

        /// <summary>
        /// Creates a new instance of PositionEventArgs specifying X, Y and Z coordinates.
        /// </summary>
        /// <param name="x">X-axis value.</param>
        /// <param name="y">Y-axis value.</param>
        /// <param name="z">Z-axis value.</param>
        public PositionEventArgs(float x, float y, float z)
            : this(x, y)
        {
            Z = z;
        }

        #endregion
    }
}
